/* Broadcom NetXtreme-C/E network driver.
 *
 * Copyright (c) 2020 Broadcom Limited
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 */
#include "bnxt.h"

#ifdef HAVE_FLOW_OFFLOAD_H
#if !defined(HAVE_FLOW_STATS_DROPS) && defined(HAVE_FLOW_ACTION_BASIC_HW_STATS_CHECK)
#define flow_stats_update(flow_stats, bytes, pkts, drops, last_used, used_hw_stats) \
	flow_stats_update(flow_stats, bytes, pkts, last_used, used_hw_stats)
#elif !defined(HAVE_FLOW_ACTION_BASIC_HW_STATS_CHECK)
#define flow_stats_update(flow_stats, bytes, pkts, drops, last_used, used_hw_stats) \
	flow_stats_update(flow_stats, bytes, pkts, last_used)
#endif

#ifndef HAVE_FLOW_ACTION_BASIC_HW_STATS_CHECK
static inline bool
flow_action_basic_hw_stats_check(const struct flow_action *action,
				 struct netlink_ext_ack *extack)
{
	return true;
}
#endif /* HAVE_FLOW_ACTION_BASIC_HW_STATS_CHECK */

#ifndef HAVE_FLOW_INDR_BLOCK_CLEANUP
#define bnxt_tc_setup_indr_block(netdev, sch, bp, f, data, cleanup)	\
	bnxt_tc_setup_indr_block(netdev, bp, f)

#define flow_indr_block_cb_alloc(cb, cb_ident, cb_priv, bnxt_tc_setup_indr_rel,	\
				 f, netdev, sch, data, bp, cleanup)			\
	flow_block_cb_alloc(cb, cb_ident, cb_priv, bnxt_tc_setup_indr_rel)

#define flow_indr_block_cb_remove(block_cb, f)	\
	flow_block_cb_remove(block_cb, f)

#define bnxt_tc_setup_indr_cb(netdev, sch, cb_priv, type, type_data, data, cleanup)	\
	bnxt_tc_setup_indr_cb(netdev, cb_priv, type, type_data)
#endif /* HAVE_FLOW_INDR_BLOCK_CLEANUP */
#endif /* HAVE_FLOW_OFFLOAD_H */

#if defined(CONFIG_BNXT_FLOWER_OFFLOAD) && defined(HAVE_FLOW_INDR_BLOCK_CB)
#if !defined(HAVE_FLOW_INDR_DEV_RGTR)
int bnxt_tc_indr_block_event(struct notifier_block *nb, unsigned long event,
			     void *ptr);

static inline int flow_indr_dev_register(flow_indr_block_bind_cb_t *cb,
					 void *cb_priv)
{
	struct bnxt *bp = cb_priv;

	bp->tc_netdev_nb.notifier_call = bnxt_tc_indr_block_event;
	return register_netdevice_notifier(&bp->tc_netdev_nb);
}

static inline void flow_indr_dev_unregister(flow_indr_block_bind_cb_t *cb,
					    void *cb_priv,
					    void (*release)(void *cb_priv))
{
	struct bnxt *bp = cb_priv;

	unregister_netdevice_notifier(&bp->tc_netdev_nb);
}
#endif /* !HAVE_FLOW_INDR_DEV_RGTR */

#ifdef HAVE_OLD_FLOW_INDR_DEV_UNRGTR
#define flow_indr_dev_unregister(cb, bp, rel)		\
	flow_indr_dev_unregister(cb, bp, bnxt_tc_setup_indr_block_cb)
#endif /* HAVE_OLD_FLOW_INDR_BLOCK_CB_UNRGTR */
#endif /* CONFIG_BNXT_FLOWER_OFFLOAD && HAVE_FLOW_INDR_BLOCK_CB */
